import { Body, CookieParam, JsonController, Post, Res } from 'routing-controllers';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { IllegalJWTError, InvalidCredentialsError, JwtNotProvidedError, PasswordNeededError, RefreshTokenCountInvalidError, UsernameOrEmailNeededError } from '../errors/AuthError';
import { UserNotFoundError } from '../errors/UserErrors';
import { CreateAuth } from '../models/actions/CreateAuth';
import { HandleLogout } from '../models/actions/HandleLogout';
import { RefreshAuth } from '../models/actions/RefreshAuth';
import { Auth } from '../models/responses/ResponseAuth';
import { Logout } from '../models/responses/ResponseLogout';

@JsonController('/auth')
export class AuthController {
	constructor() {
	}

	@Post("/login")
	@ResponseSchema(Auth)
	@ResponseSchema(InvalidCredentialsError)
	@ResponseSchema(UserNotFoundError)
	@ResponseSchema(UsernameOrEmailNeededError)
	@ResponseSchema(PasswordNeededError)
	@ResponseSchema(InvalidCredentialsError)
	@OpenAPI({ description: 'Create a new access token object' })
	async login(@Body({ validate: true }) createAuth: CreateAuth, @Res() response: any) {
		let auth;
		try {
			auth = await createAuth.toAuth();
			response.cookie('lfk_backend__refresh_token', auth.refresh_token, { expires: new Date(auth.refresh_token_expires_at * 1000), httpOnly: true });
			response.cookie('lfk_backend__refresh_token_expires_at', auth.refresh_token_expires_at, { expires: new Date(auth.refresh_token_expires_at * 1000), httpOnly: true });
			return response.send(auth)
		} catch (error) {
			throw error;
		}
	}

	@Post("/logout")
	@ResponseSchema(Logout)
	@ResponseSchema(InvalidCredentialsError)
	@ResponseSchema(UserNotFoundError)
	@ResponseSchema(UsernameOrEmailNeededError)
	@ResponseSchema(PasswordNeededError)
	@ResponseSchema(InvalidCredentialsError)
	@OpenAPI({ description: 'Create a new access token object' })
	async logout(@Body({ validate: true }) handleLogout: HandleLogout, @CookieParam("lfk_backend__refresh_token") refresh_token: string, @Res() response: any) {
		if (refresh_token && refresh_token.length != 0 && handleLogout.token == undefined) {
			handleLogout.token = refresh_token;
		}

		let logout;
		try {
			logout = await handleLogout.logout()
			await response.cookie('lfk_backend__refresh_token', "expired", { expires: new Date(Date.now()), httpOnly: true });
			response.cookie('lfk_backend__refresh_token_expires_at', "expired", { expires: new Date(Date.now()), httpOnly: true });
		} catch (error) {
			throw error;
		}
		return response.send(logout)
	}

	@Post("/refresh")
	@ResponseSchema(Auth)
	@ResponseSchema(JwtNotProvidedError)
	@ResponseSchema(IllegalJWTError)
	@ResponseSchema(UserNotFoundError)
	@ResponseSchema(RefreshTokenCountInvalidError)
	@OpenAPI({ description: 'refresh a access token' })
	async refresh(@Body({ validate: true }) refreshAuth: RefreshAuth, @CookieParam("lfk_backend__refresh_token") refresh_token: string, @Res() response: any) {
		if (refresh_token && refresh_token.length != 0 && refreshAuth.token == undefined) {
			refreshAuth.token = refresh_token;
		}
		let auth;
		try {
			auth = await refreshAuth.toAuth();
			response.cookie('lfk_backend__refresh_token', auth.refresh_token, { expires: new Date(auth.refresh_token_expires_at * 1000), httpOnly: true });
			response.cookie('lfk_backend__refresh_token_expires_at', auth.refresh_token_expires_at, { expires: new Date(auth.refresh_token_expires_at * 1000), httpOnly: true });
		} catch (error) {
			throw error;
		}
		return response.send(auth)
	}
}
