import {
  IsInt,
  IsNotEmpty,
  IsOptional,
  IsPostalCode,
  IsString
} from "class-validator";
import { Column, Entity, OneToMany, PrimaryGeneratedColumn } from "typeorm";
import { Participant } from "./Participant";
import { RunnerOrganisation } from "./RunnerOrganisation";

/**
 * Defines a address (to be used for contact information).
*/
@Entity()
export class Address {
  /**
   * Autogenerated unique id (primary key).
   */
  @PrimaryGeneratedColumn()
  @IsInt()
  id: number;

  /**
   * The address's description.
   */
  @Column({ nullable: true })
  @IsString()
  @IsOptional()
  description?: string;

  /**
   * The address's first line.
   * Containing the street and house number.
   */
  @Column()
  @IsString()
  @IsNotEmpty()
  address1: string;

  /**
   * The address's second line.
   * Containing optional information.
   */
  @Column({ nullable: true })
  @IsString()
  @IsOptional()
  address2?: string;

  /**
   * The address's postal code.
   */
  @Column()
  @IsString()
  @IsNotEmpty()
  @IsPostalCode("DE")
  postalcode: string;

  /**
   * The address's city.
   */
  @Column()
  @IsString()
  @IsNotEmpty()
  city: string;

  /**
   * The address's country.
   */
  @Column()
  @IsString()
  @IsNotEmpty()
  country: string;

  /**
   * Used to link the address to participants.
   */
  @OneToMany(() => Participant, participant => participant.address, { nullable: true })
  participants: Participant[];

  /**
   * Used to link the address to runner groups.
   */
  @OneToMany(() => RunnerOrganisation, group => group.address, { nullable: true })
  groups: RunnerOrganisation[];
}
